@@ -2,7 +2,7 @@
  * RTSP protocol handler. This file is part of Shairport.
  * Copyright (c) James Laird 2013
  * Modifications associated with audio synchronization, mutithreading and
- * metadata handling copyright (c) Mike Brady 2014-2015
+ * metadata handling copyright (c) Mike Brady 2014-2017
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
@@ -69,10 +69,13 @@
 #define INETx_ADDRSTRLEN INET_ADDRSTRLEN
 #endif
 
+#define METADATA_SNDBUF (4 * 1024 * 1024)
+
 enum rtsp_read_request_response {
   rtsp_read_request_response_ok,
-  rtsp_read_request_response_shutdown_requested,
+  rtsp_read_request_response_immediate_shutdown_requested,
   rtsp_read_request_response_bad_packet,
+  rtsp_read_request_response_channel_closed,
   rtsp_read_request_response_error
 };
 
@@ -90,21 +93,12 @@ static pthread_mutex_t reference_counter_lock = PTHREAD_MUTEX_INITIALIZER;
 // static int please_shutdown = 0;
 // static pthread_t playing_thread = 0;
 
-typedef struct {
-  int fd;
-  int authorized; // set is a password is required and has been supplied
-  stream_cfg stream;
-  SOCKADDR remote, local;
-  int stop;
-  int running;
-  pthread_t thread;
-  pthread_t player_thread;
-} rtsp_conn_info;
-
 static rtsp_conn_info *playing_conn =
     NULL; // the data structure representing the connection that has the player.
 static rtsp_conn_info **conns = NULL;
 
+int RTSP_connection_index = 0;
+
 void memory_barrier() {
   pthread_mutex_lock(&barrier_mutex);
   pthread_mutex_unlock(&barrier_mutex);
@@ -249,45 +243,21 @@ int pc_queue_get_item(pc_queue *the_queue, void *the_stuff) {
 
 void ask_other_rtsp_conversation_threads_to_stop(pthread_t except_this_thread);
 
-// determine if we are the currently playing thread
-static inline int rtsp_playing(void) {
-  if (pthread_mutex_trylock(&play_lock)) {
-    // if playing_mutex is locked...
-    // return 0 if the threads are different, non-zero if the threads are the same
-    return pthread_equal(playing_conn->thread, pthread_self());
-  } else {
-    // you actually acquired the playing_mutex, implying that there is no currently playing thread
-    // so unlock it return 0, implying you are not playing
-    pthread_mutex_unlock(&play_lock);
-    return 0;
-  }
-}
-
 void rtsp_request_shutdown_stream(void) {
   debug(1, "Request to shut down all rtsp conversation threads");
   ask_other_rtsp_conversation_threads_to_stop(0); // i.e. ask all playing threads to stop
 }
 
-// static void rtsp_take_player(void) {
-//  if (rtsp_playing())
-//    return;
-
-//  if (pthread_mutex_trylock(&playing_mutex)) {
-//    debug(1, "Request to all other playing threads to stop.");
-//    ask_other_rtsp_conversation_threads_to_stop(
-//        pthread_self()); // all threads apart from self
-//    pthread_mutex_lock(&playing_mutex);
-//  }
-//  playing_thread =
-//      pthread_self(); // make us the currently-playing thread (why?)
-//}
-
 // keep track of the threads we have spawned so we can join() them
 static int nconns = 0;
 static void track_thread(rtsp_conn_info *conn) {
   conns = realloc(conns, sizeof(rtsp_conn_info *) * (nconns + 1));
-  conns[nconns] = conn;
-  nconns++;
+  if (conns) {
+    conns[nconns] = conn;
+    nconns++;
+  } else {
+    die("could not reallocate memnory for \"conns\" in rtsp.c.");
+  }
 }
 
 static void cleanup_threads(void) {
@@ -296,9 +266,11 @@ static void cleanup_threads(void) {
   // debug(2, "culling threads.");
   for (i = 0; i < nconns;) {
     if (conns[i]->running == 0) {
+      debug(3, "found RTSP connection thread %d in a non-running state.",
+            conns[i]->connection_number);
       pthread_join(conns[i]->thread, &retval);
+      debug(3, "RTSP connection thread %d deleted...", conns[i]->connection_number);
       free(conns[i]);
-      debug(3, "one thread joined...");
       nconns--;
       if (nconns)
         conns[i] = conns[nconns];
@@ -313,7 +285,7 @@ static void cleanup_threads(void) {
 
 void ask_other_rtsp_conversation_threads_to_stop(pthread_t except_this_thread) {
   int i;
-  debug(2, "asking playing threads to stop");
+  debug(1, "asking playing threads to stop");
   for (i = 0; i < nconns; i++) {
     if (((except_this_thread == 0) || (pthread_equal(conns[i]->thread, except_this_thread) == 0)) &&
         (conns[i]->running != 0)) {
@@ -362,8 +334,12 @@ static void msg_retain(rtsp_message *msg) {
 
 static rtsp_message *msg_init(void) {
   rtsp_message *msg = malloc(sizeof(rtsp_message));
-  memset(msg, 0, sizeof(rtsp_message));
-  msg->referenceCount = 1; // from now on, any access to this must be protected with the lock
+  if (msg) {
+    memset(msg, 0, sizeof(rtsp_message));
+    msg->referenceCount = 1; // from now on, any access to this must be protected with the lock
+  } else {
+    die("can not allocate memory for an rtsp_message.");
+  }
   return msg;
 }
 
@@ -492,16 +468,16 @@ static enum rtsp_read_request_response rtsp_read_request(rtsp_conn_info *conn,
   while (msg_size < 0) {
     memory_barrier();
     if (conn->stop != 0) {
-      debug(1, "RTSP shutdown requested.");
-      reply = rtsp_read_request_response_shutdown_requested;
+      debug(3, "RTSP conversation thread %d shutdown requested.", conn->connection_number);
+      reply = rtsp_read_request_response_immediate_shutdown_requested;
       goto shutdown;
     }
     nread = read(conn->fd, buf + inbuf, buflen - inbuf);
 
     if (nread == 0) {
       // a blocking read that returns zero means eof -- implies connection closed
-      debug(3, "RTSP connection closed.");
-      reply = rtsp_read_request_response_shutdown_requested;
+      debug(3, "RTSP conversation thread %d -- connection closed.", conn->connection_number);
+      reply = rtsp_read_request_response_channel_closed;
       goto shutdown;
     }
 
@@ -509,7 +485,7 @@ static enum rtsp_read_request_response rtsp_read_request(rtsp_conn_info *conn,
       if (errno == EINTR)
         continue;
       perror("read failure");
-      reply = rtsp_read_request_response_error;
+      reply = rtsp_read_request_response_channel_closed;
       goto shutdown;
     }
     inbuf += nread;
@@ -627,7 +603,7 @@ static void msg_write_response(int fd, rtsp_message *resp) {
 }
 
 static void handle_record(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
-  // debug(1,"Handle Record");
+  debug(3, "Connection %d: RECORD", conn->connection_number);
   resp->respcode = 200;
   // I think this is for telling the client what the absolute minimum latency
   // actually is,
@@ -654,13 +630,15 @@ static void handle_record(rtsp_conn_info *conn, rtsp_message *req, rtsp_message
         rtptime = uatoi(p + 1); // unsigned integer -- up to 2^32-1
         rtptime--;
         // debug(1,"RTSP Flush Requested by handle_record: %u.",rtptime);
-        player_flush(rtptime);
+        player_flush(rtptime, conn);
       }
     }
   }
+  usleep(500000);
 }
 
 static void handle_options(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
+  debug(3, "Connection %d: OPTIONS", conn->connection_number);
   resp->respcode = 200;
   msg_add_header(resp, "Public", "ANNOUNCE, SETUP, RECORD, "
                                  "PAUSE, FLUSH, TEARDOWN, "
@@ -668,19 +646,29 @@ static void handle_options(rtsp_conn_info *conn, rtsp_message *req, rtsp_message
 }
 
 static void handle_teardown(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
-  if (!rtsp_playing())
-    debug(1, "This RTSP conversation thread doesn't think it's playing, but "
-             "it's sending a response to teardown anyway");
+  debug(3, "Connection %d: TEARDOWN", conn->connection_number);
+  // if (!rtsp_playing())
+  //  debug(1, "This RTSP connection thread (%d) doesn't think it's playing, but "
+  //           "it's sending a response to teardown anyway",conn->connection_number);
   resp->respcode = 200;
   msg_add_header(resp, "Connection", "close");
-  conn->stop = 1;
+
+  debug(3,
+        "TEARDOWN: synchronously terminating the player thread of RTSP conversation thread %d (2).",
+        conn->connection_number);
+  // if (rtsp_playing()) {
+  player_stop(conn);
+  debug(3, "TEARDOWN: successful termination of playing thread of RTSP conversation thread %d.",
+        conn->connection_number);
+  //}
 }
 
 static void handle_flush(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
-  if (!rtsp_playing())
-    debug(1, "This RTSP conversation thread doesn't think it's playing, but "
-             "it's sending a response to flush anyway");
-  char *p;
+  debug(3, "Connection %d: FLUSH", conn->connection_number);
+  //  if (!rtsp_playing())
+  //    debug(1, "This RTSP conversation thread (%d) doesn't think it's playing, but "
+  //            "it's sending a response to flush anyway",conn->connection_number);
+  char *p = NULL;
   uint32_t rtptime = 0;
   char *hdr = msg_get_header(req, "RTP-Info");
 
@@ -694,13 +682,19 @@ static void handle_flush(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *
         rtptime = uatoi(p + 1); // unsigned integer -- up to 2^32-1
     }
   }
-  // debug(1,"RTSP Flush Requested: %u.",rtptime);
-  player_flush(rtptime);
+// debug(1,"RTSP Flush Requested: %u.",rtptime);
+#ifdef CONFIG_METADATA
+  if (p)
+    send_metadata('ssnc', 'flsr', p, strlen(p), req, 1);
+  else
+    send_metadata('ssnc', 'flsr', NULL, 0, NULL, 0);
+#endif
+  player_flush(rtptime, conn); // will not crash even it there is no player thread.
   resp->respcode = 200;
 }
 
 static void handle_setup(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
-  // debug(1,"Handle Setup");
+  debug(3, "Connection %d: SETUP", conn->connection_number);
   int cport, tport;
   int lsport, lcport, ltport;
   uint32_t active_remote = 0;
@@ -800,7 +794,8 @@ static void handle_setup(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *
   tport = atoi(p);
 
   //  rtsp_take_player();
-  rtp_setup(&conn->local, &conn->remote, cport, tport, active_remote, &lsport, &lcport, &ltport);
+  rtp_setup(&conn->local, &conn->remote, cport, tport, active_remote, &lsport, &lcport, &ltport,
+            conn);
   if (!lsport)
     goto error;
   char *q;
@@ -819,7 +814,7 @@ static void handle_setup(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *
       strcat(hdr, q); // should unsplice the timing port entry
   }
 
-  player_play(&conn->stream, &conn->player_thread); // the thread better be 0
+  player_play(conn); // the thread better be 0
 
   char *resphdr = alloca(200);
   *resphdr = 0;
@@ -837,12 +832,15 @@ static void handle_setup(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *
   return;
 
 error:
-  warn("Error in setup request.");
+  warn("Error in setup request -- unlocking play lock on RTSP conversation thread %d.",
+       conn->connection_number);
+  playing_conn = NULL;
   pthread_mutex_unlock(&play_lock);
   resp->respcode = 451; // invalid arguments
 }
 
 static void handle_ignore(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
+  debug(1, "Connection thread %d: IGNORE", conn->connection_number);
   resp->respcode = 200;
 }
 
@@ -857,20 +855,8 @@ static void handle_set_parameter_parameter(rtsp_conn_info *conn, rtsp_message *r
 
     if (!strncmp(cp, "volume: ", 8)) {
       float volume = atof(cp + 8);
-      if (config.ignore_volume_control == 0) {
-        debug(2, "volume: %f\n", volume);
-        player_volume(volume);
-      }
-#ifdef CONFIG_METADATA
-      else {                    // if ignore volume is on...
-        char *dv = malloc(128); // will be freed in the metadata thread
-        if (dv) {
-          memset(dv, 0, 128);
-          snprintf(dv, 127, "%.2f,%.2f,%.2f,%.2f", volume, 0.0, 0.0, 0.0);
-          send_ssnc_metadata('pvol', dv, strlen(dv), 1);
-        }
-      }
-#endif
+      debug(3, "AirPlay request to set volume to: %f.", volume);
+      player_volume(volume, conn);
     } else
 #ifdef CONFIG_METADATA
         if (!strncmp(cp, "progress: ", 10)) {
@@ -890,8 +876,7 @@ static void handle_set_parameter_parameter(rtsp_conn_info *conn, rtsp_message *r
 #ifdef CONFIG_METADATA
 // Metadata is not used by shairport-sync.
 // Instead we send all metadata to a fifo pipe, so that other apps can listen to
-// the pipe and use
-// the metadata.
+// the pipe and use the metadata.
 
 // We use two 4-character codes to identify each piece of data and we send the
 // data itself, if any,
@@ -957,6 +942,11 @@ static void handle_set_parameter_parameter(rtsp_conn_info *conn, rtsp_message *r
 //		`clip` -- the payload is the IP number of the client, i.e. the sender of audio.
 //		Can be an IPv4 or an IPv6 number.
 
+//		A special sub-protocol is used for sending large data items over UDP
+//    If the payload exceeded 4 MB, it is chunked using the following format:
+//    "ssnc", "chnk", packet_ix, packet_counts, packet_tag, packet_type, chunked_data.
+//    Notice that the number of items is different to the standard
+
 // including a simple base64 encoder to minimise malloc/free activity
 
 // From Stack Overflow, with thanks:
@@ -1034,14 +1024,18 @@ void metadata_create(void) {
     if (metadata_sock < 0) {
       debug(1, "Could not open metadata socket");
     } else {
+      int buffer_size = METADATA_SNDBUF;
+      setsockopt(metadata_sock, SOL_SOCKET, SO_SNDBUF, &buffer_size, sizeof(buffer_size));
       bzero((char *)&metadata_sockaddr, sizeof(metadata_sockaddr));
       metadata_sockaddr.sin_family = AF_INET;
       metadata_sockaddr.sin_addr.s_addr = inet_addr(config.metadata_sockaddr);
       metadata_sockaddr.sin_port = htons(config.metadata_sockport);
-      if (!(metadata_sockmsg = malloc(config.metadata_sockmsglength))) {
+      metadata_sockmsg = malloc(config.metadata_sockmsglength);
+      if (metadata_sockmsg) {
+        memset(metadata_sockmsg, 0, config.metadata_sockmsglength);
+      } else {
         die("Could not malloc metadata socket buffer");
       }
-      memset(metadata_sockmsg, 0, config.metadata_sockmsglength);
     }
   }
 
@@ -1094,6 +1088,47 @@ void metadata_process(uint32_t type, uint32_t code, char *data, uint32_t length)
     memcpy(ptr, data, length);
     sendto(metadata_sock, metadata_sockmsg, length + 8, 0, (struct sockaddr *)&metadata_sockaddr,
            sizeof(metadata_sockaddr));
+  } else if (metadata_sock >= 0) {
+    // send metadata in numbered chunks using the protocol:
+    // ("ssnc", "chnk", packet_ix, packet_counts, packet_tag, packet_type, chunked_data)
+
+    uint32_t chunk_ix = 0;
+    uint32_t chunk_total = length / (config.metadata_sockmsglength - 24);
+    if (chunk_total * (config.metadata_sockmsglength - 24) < length) {
+      chunk_total++;
+    }
+    uint32_t remaining = length;
+    uint32_t v;
+    char *data_crsr = data;
+    do {
+      char *ptr = metadata_sockmsg;
+      memcpy(ptr, "ssncchnk", 8);
+      ptr += 8;
+      v = htonl(chunk_ix);
+      memcpy(ptr, &v, 4);
+      ptr += 4;
+      v = htonl(chunk_total);
+      memcpy(ptr, &v, 4);
+      ptr += 4;
+      v = htonl(type);
+      memcpy(ptr, &v, 4);
+      ptr += 4;
+      v = htonl(code);
+      memcpy(ptr, &v, 4);
+      ptr += 4;
+      uint32_t datalen = remaining;
+      if (datalen > config.metadata_sockmsglength - 24) {
+        datalen = config.metadata_sockmsglength - 24;
+      }
+      memcpy(ptr, data_crsr, datalen);
+      data_crsr += datalen;
+      sendto(metadata_sock, metadata_sockmsg, datalen + 24, 0,
+             (struct sockaddr *)&metadata_sockaddr, sizeof(metadata_sockaddr));
+      chunk_ix++;
+      remaining -= datalen;
+      if (remaining == 0)
+        break;
+    } while (1);
   }
 
   // readers may go away and come back
@@ -1259,11 +1294,12 @@ static void handle_set_parameter_metadata(rtsp_conn_info *conn, rtsp_message *re
 #endif
 
 static void handle_get_parameter(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
-  debug(1, "received GET_PARAMETER request.");
+  debug(3, "Connection %d: GET_PARAMETER", conn->connection_number);
   resp->respcode = 200;
 }
 
 static void handle_set_parameter(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
+  debug(3, "Connection %d: SET_PARAMETER", conn->connection_number);
   // if (!req->contentlength)
   //    debug(1, "received empty SET_PARAMETER request.");
 
@@ -1296,7 +1332,7 @@ static void handle_set_parameter(rtsp_conn_info *conn, rtsp_message *req, rtsp_m
     // not all items have RTP-time stuff in them, which is okay
 
     if (!strncmp(ct, "application/x-dmap-tagged", 25)) {
-      debug(2, "received metadata tags in SET_PARAMETER request.");
+      debug(3, "received metadata tags in SET_PARAMETER request.");
       if (p == NULL)
         debug(1, "Missing RTP-Time info for metadata");
       if (p)
@@ -1350,36 +1386,52 @@ static void handle_set_parameter(rtsp_conn_info *conn, rtsp_message *req, rtsp_m
   } else {
     debug(1, "missing Content-Type header in SET_PARAMETER request.");
   }
-
   resp->respcode = 200;
 }
 
 static void handle_announce(rtsp_conn_info *conn, rtsp_message *req, rtsp_message *resp) {
+  debug(3, "Connection %d: ANNOUNCE", conn->connection_number);
   int have_the_player = 0;
 
   // interrupt session if permitted
   if (pthread_mutex_trylock(&play_lock) == 0) {
     have_the_player = 1;
   } else {
+    if (playing_conn) {
+      debug(1, "RTSP Conversation thread %d already playing when asked by thread %d.",
+            playing_conn->connection_number, conn->connection_number);
+    } else {
+      debug(1, "play_lock locked but no playing_conn.");
+    }
     if (config.allow_session_interruption == 1) {
       // some other thread has the player ... ask it to relinquish the thread
       if (playing_conn) {
-        playing_conn->stop = 1;
-        memory_barrier();
-        pthread_kill(playing_conn->thread, SIGUSR1);
+        debug(1, "ANNOUNCE: playing connection %d being interrupted by connection %d.",
+              playing_conn->connection_number, conn->connection_number);
+        if (playing_conn == conn) {
+          debug(1, "ANNOUNCE asking to stop itself.");
+        } else {
+          playing_conn->stop = 1;
+          memory_barrier();
+          pthread_kill(playing_conn->thread, SIGUSR1);
+          usleep(
+              1000000); // here, it is possible for other connections to come in and nab the player.
+        }
       } else {
         die("Non existent the_playing_conn with play_lock enabled.");
       }
-      usleep(1000000); // here, it is possible for other connections to come in and nab the player.
       debug(1, "Try to get the player now");
       // pthread_mutex_lock(&play_lock);
       if (pthread_mutex_trylock(&play_lock) == 0)
         have_the_player = 1;
+      else
+        debug(1, "ANNOUNCE failed to get the player");
     }
   }
 
   if (have_the_player) {
-    playing_conn = conn;  // the present connection is now playing
+    playing_conn = conn; // the present connection is now playing
+    debug(3, "RTSP conversation thread %d has acquired play lock.", conn->connection_number);
     resp->respcode = 456; // 456 - Header Field Not Valid for Resource
     char *paesiv = NULL;
     char *prsaaeskey = NULL;
@@ -1452,17 +1504,22 @@ static void handle_announce(rtsp_conn_info *conn, rtsp_message *req, rtsp_messag
     int i;
     for (i = 0; i < sizeof(conn->stream.fmtp) / sizeof(conn->stream.fmtp[0]); i++)
       conn->stream.fmtp[i] = atoi(strsep(&pfmtp, " \t"));
+    // here we should check the sanity ot the fmtp values
+    // for (i = 0; i < sizeof(conn->stream.fmtp) / sizeof(conn->stream.fmtp[0]); i++)
+    //  debug(1,"  fmtp[%2d] is: %10d",i,conn->stream.fmtp[i]);
 
     char *hdr = msg_get_header(req, "X-Apple-Client-Name");
     if (hdr) {
-      debug(1, "Play connection from device named \"%s\".", hdr);
+      debug(1, "Play connection from device named \"%s\" on RTSP conversation thread %d.", hdr,
+            conn->connection_number);
 #ifdef CONFIG_METADATA
       send_metadata('ssnc', 'snam', hdr, strlen(hdr), req, 1);
 #endif
     }
     hdr = msg_get_header(req, "User-Agent");
     if (hdr) {
-      debug(1, "Play connection from user agent \"%s\".", hdr);
+      debug(1, "Play connection from user agent \"%s\" on RTSP conversation thread %d.", hdr,
+            conn->connection_number);
 #ifdef CONFIG_METADATA
       send_metadata('ssnc', 'snua', hdr, strlen(hdr), req, 1);
 #endif
@@ -1475,6 +1532,9 @@ static void handle_announce(rtsp_conn_info *conn, rtsp_message *req, rtsp_messag
 
 out:
   if (resp->respcode != 200 && resp->respcode != 453) {
+    debug(1, "Error in handling ANNOUNCE on conversation thread %d. Unlocking the play lock.",
+          conn->connection_number);
+    playing_conn = NULL;
     pthread_mutex_unlock(&play_lock);
   }
 }
@@ -1503,6 +1563,8 @@ static void apple_challenge(int fd, rtsp_message *req, rtsp_message *resp) {
 
   int chall_len;
   uint8_t *chall = base64_dec(hdr, &chall_len);
+  if (chall == NULL)
+    die("null chall in apple_challenge");
   uint8_t buf[48], *bp = buf;
   int i;
   memset(buf, 0, sizeof(buf));
@@ -1539,7 +1601,8 @@ static void apple_challenge(int fd, rtsp_message *req, rtsp_message *resp) {
 
   uint8_t *challresp = rsa_apply(buf, buflen, &resplen, RSA_MODE_AUTH);
   char *encoded = base64_enc(challresp, resplen);
-
+  if (encoded == NULL)
+    die("could not allocate memory for \"encoded\"");
   // strip the padding.
   char *padding = strchr(encoded, '=');
   if (padding)
@@ -1717,15 +1780,18 @@ static void *rtsp_conversation_thread_func(void *pconn) {
 
   rtsp_conn_info *conn = pconn;
 
+  rtp_initialise(conn);
+
   rtsp_message *req, *resp;
   char *hdr, *auth_nonce = NULL;
 
   enum rtsp_read_request_response reply;
 
-  do {
+  while (conn->stop == 0) {
     reply = rtsp_read_request(conn, &req);
     if (reply == rtsp_read_request_response_ok) {
-      debug(3, "RTSP Packet received of type \"%s\":", req->method),
+      debug(3, "RTSP thread %d received an RTSP Packet of type \"%s\":", conn->connection_number,
+            req->method),
           debug_print_msg_headers(3, req);
       resp = msg_init();
       resp->respcode = 400;
@@ -1749,39 +1815,43 @@ static void *rtsp_conversation_thread_func(void *pconn) {
           }
         }
         if (method_selected == 0)
-          debug(1, "Unrecognised and unhandled rtsp request \"%s\".", req->method);
+          debug(1, "RTSP thread %d: Unrecognised and unhandled rtsp request \"%s\".",
+                conn->connection_number, req->method);
       }
-      debug(3, "RTSP Response:");
+      debug(3, "RTSP thread %d: RTSP Response:", conn->connection_number);
       debug_print_msg_headers(3, resp);
       msg_write_response(conn->fd, resp);
       msg_free(req);
       msg_free(resp);
     } else {
-      if (reply != rtsp_read_request_response_shutdown_requested)
+      if ((reply == rtsp_read_request_response_immediate_shutdown_requested) ||
+          (reply == rtsp_read_request_response_channel_closed)) {
+        debug(3, "Synchronously terminate playing thread of RTSP conversation thread %d.",
+              conn->connection_number);
+        player_stop(conn);
+        debug(3, "Successful termination of playing thread of RTSP conversation thread %d.",
+              conn->connection_number);
+        debug(3, "Request termination of RTSP conversation thread %d.", conn->connection_number);
+        conn->stop = 1;
+      } else {
         debug(1, "rtsp_read_request error %d, packet ignored.", (int)reply);
+      }
     }
-  } while (reply != rtsp_read_request_response_shutdown_requested);
-
-  debug(1, "Closing down RTSP conversation thread...");
-  if (rtsp_playing()) {
-    player_stop(&conn->player_thread); // might be less noisy doing this first
-    rtp_shutdown();
-    // usleep(400000); // let an angel pass...
-    pthread_mutex_unlock(&play_lock);
   }
-  conn->running = 0;
+
   if (conn->fd > 0)
     close(conn->fd);
   if (auth_nonce)
     free(auth_nonce);
-  //    pthread_mutex_unlock(&playing_mutex);
-  // usleep(1000000);
-  //  } // else {
-  // debug(1, "This RTSP conversation thread doesn't think it's playing for a "
-  //         "close RTSP connection.");
-  // }
-  debug(2, "RTSP conversation thread terminated.");
+  rtp_terminate(conn);
+  if (playing_conn == conn) {
+    debug(3, "Unlocking play lock on RTSP conversation thread %d.", conn->connection_number);
+    playing_conn = NULL;
+    pthread_mutex_unlock(&play_lock);
+  }
+  debug(1, "RTSP conversation thread %d terminated.", conn->connection_number);
   //  please_shutdown = 0;
+  conn->running = 0;
   return NULL;
 }
 
@@ -1925,7 +1995,10 @@ void rtsp_listen_loop(void) {
       continue;
 
     rtsp_conn_info *conn = malloc(sizeof(rtsp_conn_info));
+    if (conn == 0)
+      die("Couldn't allocate memory for an rtsp_conn_info record.");
     memset(conn, 0, sizeof(rtsp_conn_info));
+    conn->connection_number = RTSP_connection_index++;
     socklen_t slen = sizeof(conn->remote);
 
     conn->fd = accept(acceptfd, (struct sockaddr *)&conn->remote, &slen);
@@ -1953,8 +2026,8 @@ void rtsp_listen_loop(void) {
           send_ssnc_metadata('clip', strdup(remote_ip4), strlen(remote_ip4), 1);
           send_ssnc_metadata('svip', strdup(ip4), strlen(ip4), 1);
 #endif
-          debug(1, "New RTSP connection from %s:%u to self at %s:%u.", remote_ip4, rport, ip4,
-                tport);
+          debug(1, "New RTSP connection from %s:%u to self at %s:%u on conversation thread %d.",
+                remote_ip4, rport, ip4, tport, conn->connection_number);
         }
 #ifdef AF_INET6
         if (local_info->SAFAMILY == AF_INET6) {
@@ -1974,25 +2047,25 @@ void rtsp_listen_loop(void) {
           send_ssnc_metadata('clip', strdup(remote_ip6), strlen(remote_ip6), 1);
           send_ssnc_metadata('svip', strdup(ip6), strlen(ip6), 1);
 #endif
-          debug(1, "New RTSP connection from [%s]:%u to self at [%s]:%u.", remote_ip6, rport, ip6,
-                tport);
+          debug(1, "New RTSP connection from [%s]:%u to self at [%s]:%u on conversation thread %d.",
+                remote_ip6, rport, ip6, tport, conn->connection_number);
         }
 #endif
 
       } else {
         debug(1, "Error figuring out Shairport Sync's own IP number.");
       }
-
-      usleep(500000);
-      pthread_t rtsp_conversation_thread;
-      ret = pthread_create(&rtsp_conversation_thread, NULL, rtsp_conversation_thread_func, conn);
+      //      usleep(500000);
+      //      pthread_t rtsp_conversation_thread;
+      //      conn->thread = rtsp_conversation_thread;
+      //      conn->stop = 0; // record's memory has been zeroed
+      //      conn->authorized = 0; // record's memory has been zeroed
+      ret = pthread_create(&conn->thread, NULL, rtsp_conversation_thread_func,
+                           conn); // also acts as a memory barrier
       if (ret)
-        die("Failed to create RTSP receiver thread!");
-
-      conn->thread = rtsp_conversation_thread;
-      conn->stop = 0;
-      conn->authorized = 0;
-      conn->running = 1;
+        die("Failed to create RTSP receiver thread %d!", conn->connection_number);
+      debug(3, "Successfully created RTSP receiver thread %d.", conn->connection_number);
+      conn->running = 1; // this must happen before the thread is tracked
       track_thread(conn);
     }
   }
